@@ -38,9 +38,8 @@ int _gnutls_version_priority (gnutls_session_t session,
                               gnutls_protocol_t version);
 int _gnutls_version_is_supported (gnutls_session_t session,
                                   const gnutls_protocol_t version);
-int _gnutls_version_get_major (gnutls_protocol_t ver);
-int _gnutls_version_get_minor (gnutls_protocol_t ver);
-gnutls_protocol_t _gnutls_version_get (int major, int minor);
+void _gnutls_version_to_tls (gnutls_protocol_t ver, uint8_t* major, uint8_t* minor);
+gnutls_protocol_t _gnutls_version_get (uint8_t major, uint8_t minor);
 
 /* Functions for feature checks */
 int _gnutls_version_has_selectable_prf (gnutls_protocol_t version);
@@ -50,7 +49,6 @@ int _gnutls_version_has_explicit_iv (gnutls_protocol_t version);
 
 /* Functions for MACs. */
 int _gnutls_mac_is_ok (gnutls_mac_algorithm_t algorithm);
-#define _gnutls_x509_oid_to_mac(oid) (gnutls_mac_algorithm_t)_gnutls_x509_oid_to_digest(oid)
 gnutls_digest_algorithm_t _gnutls_x509_oid_to_digest (const char *oid);
 const char *_gnutls_x509_mac_to_oid (gnutls_mac_algorithm_t mac);
 
@@ -90,12 +88,10 @@ _gnutls_cipher_suite_get_id (gnutls_kx_algorithm_t kx_algorithm,
 int _gnutls_cipher_is_block (gnutls_cipher_algorithm_t algorithm);
 int _gnutls_cipher_algo_is_aead (gnutls_cipher_algorithm_t algorithm);
 int _gnutls_cipher_is_ok (gnutls_cipher_algorithm_t algorithm);
-int _gnutls_cipher_get_iv_size (gnutls_cipher_algorithm_t algorithm);
-int _gnutls_cipher_get_export_flag (gnutls_cipher_algorithm_t algorithm);
+int _gnutls_cipher_get_tag_size (gnutls_cipher_algorithm_t algorithm);
 
 /* Functions for key exchange. */
 int _gnutls_kx_needs_dh_params (gnutls_kx_algorithm_t algorithm);
-int _gnutls_kx_needs_rsa_params (gnutls_kx_algorithm_t algorithm);
 mod_auth_st *_gnutls_kx_auth_struct (gnutls_kx_algorithm_t algorithm);
 int _gnutls_kx_is_ok (gnutls_kx_algorithm_t algorithm);
 
@@ -145,19 +141,6 @@ struct gnutls_ecc_curve_entry_st
   gnutls_ecc_curve_t id;
   int tls_id; /* The RFC4492 namedCurve ID */
   int size; /* the size in bytes */
-
-  /** The prime that defines the field the curve is in (encoded in hex) */
-  const char *prime;
-  /** The fields A param (hex) */
-  const char *A;
-  /** The fields B param (hex) */
-  const char *B;
-  /** The order of the curve (hex) */
-  const char *order;
-  /** The x co-ordinate of the base point on the curve (hex) */
-  const char *Gx;
-  /** The y co-ordinate of the base point on the curve (hex) */
-  const char *Gy;
 };
 typedef struct gnutls_ecc_curve_entry_st gnutls_ecc_curve_entry_st;
 
