@@ -72,6 +72,7 @@ typedef struct
 {
   int session_ticket_enable;
   int session_ticket_renew;
+  uint8_t session_ticket_IV[SESSION_TICKET_IV_SIZE];
 
   uint8_t *session_ticket;
   int session_ticket_len;
@@ -92,24 +93,24 @@ static int
 digest_ticket (const gnutls_datum_t * key, struct ticket *ticket,
                uint8_t * digest)
 {
-  digest_hd_st digest_hd;
+  mac_hd_st digest_hd;
   uint16_t length16;
   int ret;
 
-  ret = _gnutls_hmac_init (&digest_hd, GNUTLS_MAC_SHA256, key->data,
+  ret = _gnutls_mac_init (&digest_hd, GNUTLS_MAC_SHA256, key->data,
                            key->size);
   if (ret < 0)
     {
       gnutls_assert ();
       return ret;
     }
-  _gnutls_hmac (&digest_hd, ticket->key_name, KEY_NAME_SIZE);
-  _gnutls_hmac (&digest_hd, ticket->IV, IV_SIZE);
+  _gnutls_mac (&digest_hd, ticket->key_name, KEY_NAME_SIZE);
+  _gnutls_mac (&digest_hd, ticket->IV, IV_SIZE);
   length16 = _gnutls_conv_uint16 (ticket->encrypted_state_len);
-  _gnutls_hmac (&digest_hd, &length16, 2);
-  _gnutls_hmac (&digest_hd, ticket->encrypted_state,
+  _gnutls_mac (&digest_hd, &length16, 2);
+  _gnutls_mac (&digest_hd, ticket->encrypted_state,
                 ticket->encrypted_state_len);
-  _gnutls_hmac_deinit (&digest_hd, digest);
+  _gnutls_mac_deinit (&digest_hd, digest);
 
   return 0;
 }
@@ -190,7 +191,6 @@ encrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
 {
   cipher_hd_st cipher_hd;
   gnutls_datum_t key, IV, mac_secret, state, encrypted_state;
-  uint8_t iv[IV_SIZE];
   int blocksize;
   int ret;
 
@@ -219,17 +219,8 @@ encrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
   /* Encrypt state using 128-bit AES in CBC mode. */
   key.data = (void *) &priv->key[KEY_POS];
   key.size = KEY_SIZE;
-  IV.data = iv;
+  IV.data = priv->session_ticket_IV;
   IV.size = IV_SIZE;
-
-  ret = _gnutls_rnd (GNUTLS_RND_NONCE, iv, IV_SIZE);
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      _gnutls_free_datum (&encrypted_state);
-      return ret;
-    }
-
   ret =
     _gnutls_cipher_init (&cipher_hd, GNUTLS_CIPHER_AES_128_CBC, &key, &IV, 1);
   if (ret < 0)
@@ -545,6 +536,7 @@ int
 gnutls_session_ticket_enable_server (gnutls_session_t session,
                                      const gnutls_datum_t * key)
 {
+  int ret;
   session_ticket_ext_st *priv = NULL;
   extension_priv_data_t epriv;
 
@@ -563,6 +555,13 @@ gnutls_session_ticket_enable_server (gnutls_session_t session,
     }
   epriv.ptr = priv;
 
+  ret = _gnutls_rnd (GNUTLS_RND_NONCE, priv->session_ticket_IV, IV_SIZE);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      return ret;
+    }
+
   memcpy (&priv->key, key->data, key->size);
   priv->session_ticket_enable = 1;
 
