@@ -40,6 +40,7 @@
 #include <ext/ecc.h>
 #include <ext/status_request.h>
 #include <ext/srtp.h>
+#include <ext/alpn.h>
 #include <ext/new_record_padding.h>
 #include <gnutls_num.h>
 
@@ -368,6 +369,12 @@ _gnutls_ext_init (void)
     return ret;
 #endif
 
+#ifdef ENABLE_ALPN
+  ret = _gnutls_ext_register (&ext_mod_alpn);
+  if (ret != GNUTLS_E_SUCCESS)
+    return ret;
+#endif
+
   return GNUTLS_E_SUCCESS;
 }
 
