@@ -205,15 +205,13 @@ gnutls_compression_set_priority (gnutls_session_t session, const int *list)
 int
 gnutls_protocol_set_priority (gnutls_session_t session, const int *list)
 {
+  _set_priority (&session->internals.priorities.protocol, list);
 
   /* set the current version to the first in the chain.
    * This will be overridden later.
    */
   if (list)
-    {
-      _set_priority (&session->internals.priorities.protocol, list);
-      _gnutls_set_current_version (session, list[0]);
-    }
+    _gnutls_set_current_version (session, list[0]);
 
   return 0;
 }
@@ -284,11 +282,13 @@ static const int protocol_priority[] = {
   GNUTLS_TLS1_1,
   GNUTLS_TLS1_0,
   GNUTLS_SSL3,
+  GNUTLS_DTLS1_2,
   GNUTLS_DTLS1_0,
   0
 };
 
 static const int dtls_protocol_priority[] = {
+  GNUTLS_DTLS1_2,
   GNUTLS_DTLS1_0,
   0
 };
@@ -311,18 +311,6 @@ static const int kx_priority_performance[] = {
   0
 };
 
-static const int kx_priority_pfs[] = {
-#ifdef ENABLE_ECDHE
-  GNUTLS_KX_ECDHE_ECDSA,
-  GNUTLS_KX_ECDHE_RSA,
-#endif
-#ifdef ENABLE_DHE
-  GNUTLS_KX_DHE_RSA,
-  GNUTLS_KX_DHE_DSS,
-#endif
-  0
-};
-
 static const int kx_priority_suiteb[] = {
   GNUTLS_KX_ECDHE_ECDSA,
   0
@@ -337,9 +325,6 @@ static const int kx_priority_export[] = {
 #ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
-#endif
-#ifdef ENABLE_RSA_EXPORT
-  GNUTLS_KX_RSA_EXPORT,
 #endif
   0
 };
@@ -362,27 +347,53 @@ static const int kx_priority_secure[] = {
   GNUTLS_KX_DHE_DSS,
 #endif
   /* GNUTLS_KX_ANON_DH: Man-in-the-middle prone, don't add!
-   * GNUTLS_KX_RSA_EXPORT: Deprecated, don't add!
    */
   0
 };
 
-static const int cipher_priority_performance[] = {
+static const int cipher_priority_performance_sw[] = {
   GNUTLS_CIPHER_ARCFOUR_128,
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
   GNUTLS_CIPHER_3DES_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_256_GCM,
+  0
+};
+
+/* If GCM and AES acceleration is available then prefer
+ * them over anything else.
+ */
+static const int cipher_priority_performance_hw_aes[] = {
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_128_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
+  GNUTLS_CIPHER_AES_256_CBC,
+  GNUTLS_CIPHER_ARCFOUR_128,
+  GNUTLS_CIPHER_CAMELLIA_128_CBC,
+  GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_3DES_CBC,
   0
 };
 
-static const int cipher_priority_normal[] = {
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_GCM,
+static const int cipher_priority_normal_sw[] = {
   GNUTLS_CIPHER_AES_128_CBC,
+  GNUTLS_CIPHER_CAMELLIA_128_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_256_CBC,
+  GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
+  GNUTLS_CIPHER_3DES_CBC,
+  GNUTLS_CIPHER_ARCFOUR_128,
+  0
+};
+
+static const int cipher_priority_normal_hw_aes[] = {
+  GNUTLS_CIPHER_AES_128_GCM,
+  GNUTLS_CIPHER_AES_128_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
@@ -391,6 +402,10 @@ static const int cipher_priority_normal[] = {
   0
 };
 
+static const int *cipher_priority_performance = cipher_priority_performance_sw;
+static const int *cipher_priority_normal = cipher_priority_normal_sw;
+
+
 static const int cipher_priority_suiteb128[] = {
   GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_256_GCM,
@@ -404,32 +419,30 @@ static const int cipher_priority_suiteb192[] = {
 
 
 static const int cipher_priority_secure128[] = {
-  GNUTLS_CIPHER_AES_128_GCM,
-  GNUTLS_CIPHER_AES_256_GCM,
-
   GNUTLS_CIPHER_AES_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
   0
 };
 
 
 static const int cipher_priority_secure192[] = {
-  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_256_GCM,
   0
 };
 
 /* The same as cipher_priority_security_normal + arcfour-40. */
 static const int cipher_priority_export[] = {
-  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_AES_128_CBC,
-  GNUTLS_CIPHER_AES_256_GCM,
   GNUTLS_CIPHER_AES_256_CBC,
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
+  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_3DES_CBC,
   GNUTLS_CIPHER_ARCFOUR_128,
   GNUTLS_CIPHER_ARCFOUR_40,
@@ -543,18 +556,21 @@ typedef void (rmadd_func) (priority_st * priority_list, unsigned int alg);
 static void
 prio_remove (priority_st * priority_list, unsigned int algo)
 {
-  unsigned int i;
+  int i = 0;
+  int pos = -1;                 /* the position of the cipher to remove */
 
-  for (i=0;i<priority_list->algorithms;i++)
+  while (priority_list->priority[i] != 0)
     {
-      if (priority_list->priority[i] == algo) 
-        {
-          priority_list->algorithms--;
-          if ((priority_list->algorithms-i) > 0)
-            memmove(&priority_list->priority[i], &priority_list->priority[i+1], (priority_list->algorithms-i)*sizeof(priority_list->priority[0]));
-          priority_list->priority[priority_list->algorithms] = 0;
-          break;
-        }
+      if (priority_list->priority[i] == algo)
+        pos = i;
+      i++;
+    }
+
+  if (pos >= 0)
+    {
+      priority_list->priority[pos] = priority_list->priority[i - 1];
+      priority_list->priority[i - 1] = 0;
+      priority_list->algorithms--;
     }
 
   return;
@@ -563,19 +579,19 @@ prio_remove (priority_st * priority_list, unsigned int algo)
 static void
 prio_add (priority_st * priority_list, unsigned int algo)
 {
-  unsigned int i, l = priority_list->algorithms;
-
-  if (l >= MAX_ALGOS)
-    return;                     /* can't add it anyway */
-
-  for (i = 0; i < l; ++i)
+  register int i = 0;
+  while (priority_list->priority[i] != 0)
     {
       if (algo == priority_list->priority[i])
         return;                 /* if it exists */
+      i++;
     }
 
-  priority_list->priority[l] = algo;
-  priority_list->algorithms++;
+  if (i < MAX_ALGOS)
+    {
+      priority_list->priority[i] = algo;
+      priority_list->algorithms++;
+    }
 
   return;
 }
@@ -626,7 +642,6 @@ gnutls_priority_set (gnutls_session_t session, gnutls_priority_t priority)
 
 #define LEVEL_NONE "NONE"
 #define LEVEL_NORMAL "NORMAL"
-#define LEVEL_PFS "PFS"
 #define LEVEL_PERFORMANCE "PERFORMANCE"
 #define LEVEL_SECURE128 "SECURE128"
 #define LEVEL_SECURE192 "SECURE192"
@@ -635,10 +650,6 @@ gnutls_priority_set (gnutls_session_t session, gnutls_priority_t priority)
 #define LEVEL_SUITEB192 "SUITEB192"
 #define LEVEL_EXPORT "EXPORT"
 
-#define SET_LEVEL(to_set) \
-	if (priority_cache->level == 0 || priority_cache->level > to_set) \
-		priority_cache->level = to_set
-
 static
 int check_level(const char* level, gnutls_priority_t priority_cache, int add)
 {
@@ -657,7 +668,8 @@ bulk_rmadd_func *func;
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
 
-      SET_LEVEL(GNUTLS_SEC_PARAM_VERY_WEAK);
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_NORMAL) == 0)
@@ -669,19 +681,8 @@ bulk_rmadd_func *func;
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
 
-      SET_LEVEL(GNUTLS_SEC_PARAM_VERY_WEAK);
-      return 1;
-    }
-  else if (strcasecmp (level, LEVEL_PFS) == 0)
-    {
-      func (&priority_cache->cipher, cipher_priority_normal);
-      func (&priority_cache->kx, kx_priority_pfs);
-      func (&priority_cache->mac, mac_priority_normal);
-      func (&priority_cache->sign_algo,
-                     sign_priority_default);
-      func (&priority_cache->supported_ecc, supported_ecc_normal);
-
-      SET_LEVEL(GNUTLS_SEC_PARAM_VERY_WEAK);
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SECURE256) == 0
@@ -696,7 +697,8 @@ bulk_rmadd_func *func;
       func (&priority_cache->supported_ecc, supported_ecc_secure192);
       
       /* be conservative for now. Set the bits to correspond to 96-bit level */
-      SET_LEVEL(GNUTLS_SEC_PARAM_LEGACY);
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_LEGACY;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SECURE128) == 0
@@ -711,7 +713,8 @@ bulk_rmadd_func *func;
       func (&priority_cache->supported_ecc, supported_ecc_secure128);
 
       /* be conservative for now. Set the bits to correspond to an 72-bit level */
-      SET_LEVEL(GNUTLS_SEC_PARAM_WEAK);
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_WEAK;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SUITEB128) == 0)
@@ -725,7 +728,8 @@ bulk_rmadd_func *func;
                      sign_priority_suiteb128);
       func (&priority_cache->supported_ecc, supported_ecc_suiteb128);
 
-      SET_LEVEL(GNUTLS_SEC_PARAM_HIGH);
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_HIGH;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SUITEB192) == 0)
@@ -739,7 +743,8 @@ bulk_rmadd_func *func;
                      sign_priority_suiteb192);
       func (&priority_cache->supported_ecc, supported_ecc_suiteb192);
 
-      SET_LEVEL(GNUTLS_SEC_PARAM_ULTRA);
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_ULTRA;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_EXPORT) == 0)
@@ -751,7 +756,8 @@ bulk_rmadd_func *func;
                      sign_priority_default);
       func (&priority_cache->supported_ecc, supported_ecc_normal);
 
-      SET_LEVEL(GNUTLS_SEC_PARAM_EXPORT);
+      if (priority_cache->level == 0)
+        priority_cache->level = GNUTLS_SEC_PARAM_EXPORT;
       return 1;
     }
   return 0;
@@ -779,10 +785,6 @@ bulk_rmadd_func *func;
  * included as a fallback only.  The ciphers are sorted by security
  * margin.
  *
- * "PFS" means all "secure" ciphersuites that support perfect forward secrecy. 
- * The 256-bit ciphers are included as a fallback only.  
- * The ciphers are sorted by security margin.
- *
  * "SECURE128" means all "secure" ciphersuites of security level 128-bit
  * or more.
  *
@@ -836,7 +838,6 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
   int broken_list_size = 0, i = 0, j;
   char *darg = NULL;
   int algo;
-  unsigned ikeyword_set = 0;
   rmadd_func *fn;
   bulk_rmadd_func *bulk_fn;
 
@@ -880,19 +881,18 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
       _set_priority (&(*priority_cache)->cert_type, cert_type_priority_default);
       _set_priority (&(*priority_cache)->sign_algo, sign_priority_default);
       _set_priority (&(*priority_cache)->supported_ecc, supported_ecc_normal);
+      (*priority_cache)->level = GNUTLS_SEC_PARAM_VERY_WEAK;
       i = 0;
     }
   else
     {
       i = 1;
-      ikeyword_set = 1;
     }
 
   for (; i < broken_list_size; i++)
     {
-      if (check_level(broken_list[i], *priority_cache, ikeyword_set) != 0)
+      if (check_level(broken_list[i], *priority_cache, 0) != 0)
         {
-          ikeyword_set = 1;
           continue;
         }
       else if (broken_list[i][0] == '!' || broken_list[i][0] == '+'
@@ -1118,7 +1118,6 @@ error:
     }
   gnutls_free (darg);
   gnutls_free (*priority_cache);
-  *priority_cache = NULL;
 
   return GNUTLS_E_INVALID_REQUEST;
 
@@ -1261,6 +1260,15 @@ gnutls_set_default_export_priority (gnutls_session_t session)
   return gnutls_priority_set_direct (session, "EXPORT", NULL);
 }
 
+/* Increases the priority of AES-GCM as it is much faster
+ * than anything else if hardware support is there.
+ */
+void _gnutls_priority_prefer_aes_gcm(void)
+{
+  cipher_priority_performance = cipher_priority_performance_hw_aes;
+  cipher_priority_normal = cipher_priority_normal_hw_aes;
+}
+
 /**
  * gnutls_priority_ecc_curve_list:
  * @pcache: is a #gnutls_prioritity_t structure.
