@@ -521,13 +521,8 @@ gnutls_pubkey_get_openpgp_key_id (gnutls_pubkey_t key, unsigned int flags,
   if (key->openpgp_key_id_set == 0)
     return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-  if (subkey)
-    {
-      if (key->openpgp_key_id_set == OPENPGP_KEY_SUBKEY)
-        *subkey = 1;
-      else
-        *subkey = 0;
-    }
+  if (key->openpgp_key_id_set == OPENPGP_KEY_SUBKEY)
+    if (subkey) *subkey = 1;
 
   if (output_data)
     {
@@ -1378,10 +1373,6 @@ gnutls_pubkey_import_ecc_raw (gnutls_pubkey_t key,
 
   key->params.flags = curve;
 
-  ret = _gnutls_ecc_curve_fill_params(curve, &key->params);
-  if (ret < 0)
-    return gnutls_assert_val(ret);
-
   if (_gnutls_mpi_scan_nz (&key->params.params[ECC_X], x->data, x->size))
     {
       gnutls_assert ();
