@@ -500,9 +500,6 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
   else
     send_data_size = data_size;
   
-  if (unlikely(send_data_size == 0))
-    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
-
   /* Only encrypt if we don't have data to send 
    * from the previous run. - probably interrupted.
    */
@@ -518,6 +515,9 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
     }
   else
     {
+      if (unlikely(send_data_size == 0))
+        return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
       /* now proceed to packet encryption
        */
       cipher_size = MAX_RECORD_SEND_SIZE(session);
@@ -647,32 +647,71 @@ inline static int
 record_check_version (gnutls_session_t session,
                       gnutls_handshake_description_t htype, uint8_t version[2])
 {
-  if (htype == GNUTLS_HANDSHAKE_CLIENT_HELLO)
+unsigned vers = gnutls_protocol_get_version (session);
+int diff = (vers !=
+           _gnutls_version_get (version[0], version[1]));
+
+  if (!IS_DTLS(session))
     {
-      /* Reject hello packets with major version higher than 3.
-       */
-      if (!(IS_DTLS(session)) && version[0] > 3)
+      if (htype == GNUTLS_HANDSHAKE_CLIENT_HELLO ||
+          htype == GNUTLS_HANDSHAKE_SERVER_HELLO)
         {
+          if (version[0] != 3)
+            {
+              gnutls_assert ();
+              _gnutls_record_log
+                ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n", session,
+                 htype, version[0], version[1]);
+              return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+            }
+        }
+      else if (diff != 0)
+        {
+          /* Reject record packets that have a different version than the
+           * one negotiated. Note that this version is not protected by any
+           * mac. I don't really think that this check serves any purpose.
+           */
           gnutls_assert ();
-          _gnutls_record_log
-            ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n", session,
-             htype, version[0], version[1]);
+          _gnutls_record_log ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n",
+                          session, htype, version[0], version[1]);
+
           return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
         }
     }
-  else if (htype != GNUTLS_HANDSHAKE_SERVER_HELLO &&
-           gnutls_protocol_get_version (session) !=
-           _gnutls_version_get (version[0], version[1]))
+  else /* DTLS */
     {
-      /* Reject record packets that have a different version than the
-       * one negotiated. Note that this version is not protected by any
-       * mac. I don't really think that this check serves any purpose.
+      /* In DTLS the only information we have here is whether we
+       * expect a handshake message or not.
        */
-      gnutls_assert ();
-      _gnutls_record_log ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n",
-                          session, htype, version[0], version[1]);
+      if (htype == (gnutls_handshake_description_t)-1)
+        {
+          if (diff)
+            {
+              /* Reject record packets that have a different version than the
+               * one negotiated. Note that this version is not protected by any
+               * mac. I don't really think that this check serves any purpose.
+               */
+              gnutls_assert ();
+              _gnutls_record_log ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n",
+                              session, htype, version[0], version[1]);
 
-      return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+              return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+            }
+        }
+      else if (vers > GNUTLS_DTLS1_0 && version[0] > 254)
+        {
+          gnutls_assert ();
+          _gnutls_record_log("REC[%p]: INVALID DTLS VERSION PACKET: (%d) %d.%d\n", session,
+                 htype, version[0], version[1]);
+          return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+        }
+      else if (vers == GNUTLS_DTLS0_9 && version[0] > 1)
+        {
+          gnutls_assert ();
+          _gnutls_record_log("REC[%p]: INVALID DTLS VERSION PACKET: (%d) %d.%d\n", session,
+                 htype, version[0], version[1]);
+          return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+        }
     }
 
   return 0;
@@ -1186,7 +1225,8 @@ begin:
       ret = _dtls_record_check(record_params, packet_sequence);
       if (ret < 0)
         {
-          _gnutls_audit_log(session, "Discarded duplicate message[%u]: %s\n",
+          _gnutls_audit_log(session, "Discarded duplicate message[%u.%u]: %s\n",
+            (unsigned int)record.sequence.i[0]*256 +(unsigned int)record.sequence.i[1],
             (unsigned int) _gnutls_uint64touint32 (packet_sequence), _gnutls_packet2str (record.type));
           goto sanity_check_error;
         }
