@@ -42,12 +42,10 @@
 #include <auth/anon.h>
 #include <auth/psk.h>
 #include <algorithms.h>
-#include <gnutls_rsa_export.h>
 #include <gnutls_extensions.h>
 #include <system.h>
 #include <random.h>
 #include <gnutls/dtls.h>
-#include <timespec.h>
 
 /* These should really be static, but src/tests.c calls them.  Make
    them public functions?  */
@@ -235,11 +233,6 @@ deinit_internal_params (gnutls_session_t session)
     gnutls_dh_params_deinit (session->internals.params.dh_params);
 #endif
 
-#ifdef ENABLE_RSA_EXPORT
-  if (session->internals.params.free_rsa_params)
-    gnutls_rsa_params_deinit (session->internals.params.rsa_params);
-#endif
-
   _gnutls_handshake_hash_buffers_clear (session);
 
   memset (&session->internals.params, 0, sizeof (session->internals.params));
@@ -269,6 +262,7 @@ _gnutls_handshake_internal_state_init (gnutls_session_t session)
   
   session->internals.dtls.hsk_read_seq = 0;
   session->internals.dtls.hsk_write_seq = 0;
+  gettime(&session->internals.dtls.handshake_start_time);
 }
 
 void
@@ -590,45 +584,6 @@ _gnutls_dh_set_secret_bits (gnutls_session_t session, unsigned bits)
   return 0;
 }
 
-/* This function will set in the auth info structure the
- * RSA exponent and the modulus.
- */
-int
-_gnutls_rsa_export_set_pubkey (gnutls_session_t session,
-                               bigint_t exponent, bigint_t modulus)
-{
-  cert_auth_info_t info;
-  int ret;
-
-  info = _gnutls_get_auth_info (session);
-  if (info == NULL)
-    return GNUTLS_E_INTERNAL_ERROR;
-
-  if (info->rsa_export.modulus.data)
-    _gnutls_free_datum (&info->rsa_export.modulus);
-
-  if (info->rsa_export.exponent.data)
-    _gnutls_free_datum (&info->rsa_export.exponent);
-
-  ret = _gnutls_mpi_dprint_lz (modulus, &info->rsa_export.modulus);
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      return ret;
-    }
-
-  ret = _gnutls_mpi_dprint_lz (exponent, &info->rsa_export.exponent);
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      _gnutls_free_datum (&info->rsa_export.modulus);
-      return ret;
-    }
-
-  return 0;
-}
-
-
 /* Sets the prime and the generator in the auth info structure.
  */
 int
@@ -797,7 +752,7 @@ _gnutls_cal_PRF_A (gnutls_mac_algorithm_t algorithm,
 {
   int ret;
 
-  ret = _gnutls_hmac_fast (algorithm, secret, secret_size, seed, seed_size, result);
+  ret = _gnutls_mac_fast (algorithm, secret, secret_size, seed, seed_size, result);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
@@ -816,7 +771,7 @@ P_hash (gnutls_mac_algorithm_t algorithm,
                 int total_bytes, uint8_t * ret)
 {
 
-  digest_hd_st td2;
+  mac_hd_st td2;
   int i, times, how, blocksize, A_size;
   uint8_t final[MAX_HASH_SIZE], Atmp[MAX_SEED_SIZE];
   int output_bytes, result;
@@ -827,7 +782,7 @@ P_hash (gnutls_mac_algorithm_t algorithm,
       return GNUTLS_E_INTERNAL_ERROR;
     }
 
-  blocksize = _gnutls_hmac_get_algo_len (algorithm);
+  blocksize = _gnutls_mac_get_algo_len (algorithm);
 
   output_bytes = 0;
   do
@@ -845,7 +800,7 @@ P_hash (gnutls_mac_algorithm_t algorithm,
 
   for (i = 0; i < times; i++)
     {
-      result = _gnutls_hmac_init (&td2, algorithm, secret, secret_size);
+      result = _gnutls_mac_init (&td2, algorithm, secret, secret_size);
       if (result < 0)
         {
           gnutls_assert ();
@@ -858,15 +813,15 @@ P_hash (gnutls_mac_algorithm_t algorithm,
                               A_size, Atmp)) < 0)
         {
           gnutls_assert ();
-          _gnutls_hmac_deinit (&td2, final);
+          _gnutls_mac_deinit (&td2, final);
           return result;
         }
 
       A_size = blocksize;
 
-      _gnutls_hmac (&td2, Atmp, A_size);
-      _gnutls_hmac (&td2, seed, seed_size);
-      _gnutls_hmac_deinit (&td2, final);
+      _gnutls_mac (&td2, Atmp, A_size);
+      _gnutls_mac (&td2, seed, seed_size);
+      _gnutls_mac_deinit (&td2, final);
 
       if ((1 + i) * blocksize < total_bytes)
         {
@@ -1138,27 +1093,6 @@ gnutls_session_resumption_requested(gnutls_session_t session)
     }
 }
 
-/*-
- * _gnutls_session_is_export - Used to check whether this session is of export grade
- * @session: is a #gnutls_session_t structure.
- *
- * This function will return non zero if this session is of export grade.
- -*/
-int
-_gnutls_session_is_export (gnutls_session_t session)
-{
-  gnutls_cipher_algorithm_t cipher;
-
-  cipher =
-    _gnutls_cipher_suite_get_cipher_algo (session->
-                                          security_parameters.cipher_suite);
-
-  if (_gnutls_cipher_get_export_flag (cipher) != 0)
-    return 1;
-
-  return 0;
-}
-
 /*-
  * _gnutls_session_is_psk - Used to check whether this session uses PSK kx
  * @session: is a #gnutls_session_t structure.
