@@ -768,36 +768,7 @@ gnutls_rsa_export_get_pubkey (gnutls_session_t session,
                               gnutls_datum_t * exponent,
                               gnutls_datum_t * modulus)
 {
-  cert_auth_info_t info;
-  int ret;
-
-  if (gnutls_auth_get_type (session) == GNUTLS_CRD_CERTIFICATE)
-    {
-      info = _gnutls_get_auth_info (session);
-      if (info == NULL)
-        return GNUTLS_E_INTERNAL_ERROR;
-
-      ret = _gnutls_set_datum (modulus, info->rsa_export.modulus.data,
-                               info->rsa_export.modulus.size);
-      if (ret < 0)
-        {
-          gnutls_assert ();
-          return ret;
-        }
-
-      ret = _gnutls_set_datum (exponent, info->rsa_export.exponent.data,
-                               info->rsa_export.exponent.size);
-      if (ret < 0)
-        {
-          gnutls_assert ();
-          _gnutls_free_datum (modulus);
-          return ret;
-        }
-
-      return 0;
-    }
-
-  return GNUTLS_E_INVALID_REQUEST;
+  return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
 }
 
 /**
@@ -812,13 +783,7 @@ gnutls_rsa_export_get_pubkey (gnutls_session_t session,
 int
 gnutls_rsa_export_get_modulus_bits (gnutls_session_t session)
 {
-  cert_auth_info_t info;
-
-  info = _gnutls_get_auth_info (session);
-  if (info == NULL)
-    return GNUTLS_E_INTERNAL_ERROR;
-
-  return mpi_buf2bits (&info->rsa_export.modulus);
+  return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
 }
 
 /**
@@ -834,7 +799,7 @@ void
 gnutls_certificate_set_rsa_export_params (gnutls_certificate_credentials_t
                                           res, gnutls_rsa_params_t rsa_params)
 {
-  res->rsa_params = rsa_params;
+  return;
 }
 #endif
 
@@ -904,3 +869,35 @@ gnutls_session_get_desc (gnutls_session_t session)
             
     return desc;
 }
+
+/**
+ * gnutls_session_set_id:
+ * @session: is a #gnutls_session_t structure.
+ * @sid: the session identifier
+ *
+ * This function sets the session ID to be used in a client hello.
+ * This is a function intended for exceptional uses. Do not use this
+ * function unless you are implementing a custom protocol.
+ *
+ * To set session resumption parameters use gnutls_session_set_data() instead.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
+ *   an error code is returned.
+ **/
+int
+gnutls_session_set_id (gnutls_session_t session, 
+                              const gnutls_datum_t * sid)
+{
+  if (session->security_parameters.entity == GNUTLS_SERVER ||
+      sid->size > TLS_MAX_SESSION_ID_SIZE)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  memset (&session->internals.resumed_security_parameters, 0,
+          sizeof (session->internals.resumed_security_parameters));
+
+  session->internals.resumed_security_parameters.session_id_size = sid->size;
+  memcpy(session->internals.resumed_security_parameters.session_id,
+  	sid->data, sid->size);
+
+  return 0;
+}
