@@ -871,7 +871,7 @@ gnutls_pkcs12_generate_mac (gnutls_pkcs12_t pkcs12, const char *pass)
   uint8_t salt[8], key[20];
   int result;
   const int iter = 1;
-  digest_hd_st td1;
+  mac_hd_st td1;
   gnutls_datum_t tmp = { NULL, 0 };
   uint8_t sha_mac[20];
 
@@ -918,7 +918,7 @@ gnutls_pkcs12_generate_mac (gnutls_pkcs12_t pkcs12, const char *pass)
 
   /* Generate the key.
    */
-  result = _gnutls_pkcs12_string_to_key (GNUTLS_MAC_SHA1, 3 /*MAC*/, salt, sizeof (salt),
+  result = _gnutls_pkcs12_string_to_key (3 /*MAC*/, salt, sizeof (salt),
                                          iter, pass, sizeof (key), key);
   if (result < 0)
     {
@@ -937,17 +937,17 @@ gnutls_pkcs12_generate_mac (gnutls_pkcs12_t pkcs12, const char *pass)
 
   /* MAC the data
    */
-  result = _gnutls_hmac_init (&td1, GNUTLS_MAC_SHA1, key, sizeof (key));
+  result = _gnutls_mac_init (&td1, GNUTLS_MAC_SHA1, key, sizeof (key));
   if (result < 0)
     {
       gnutls_assert ();
       goto cleanup;
     }
 
-  _gnutls_hmac (&td1, tmp.data, tmp.size);
+  _gnutls_mac (&td1, tmp.data, tmp.size);
   _gnutls_free_datum (&tmp);
 
-  _gnutls_hmac_deinit (&td1, sha_mac);
+  _gnutls_mac_deinit (&td1, sha_mac);
 
 
   result =
@@ -1001,20 +1001,16 @@ cleanup:
 int
 gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
 {
-  uint8_t key[MAX_HASH_SIZE];
-  char oid[MAX_OID_SIZE];
+  uint8_t key[20];
   int result;
   unsigned int iter;
   int len;
-  digest_hd_st td1;
+  mac_hd_st td1;
   gnutls_datum_t tmp = { NULL, 0 }, salt =
   {
   NULL, 0};
-  uint8_t mac_output[MAX_HASH_SIZE];
-  uint8_t mac_output_orig[MAX_HASH_SIZE];
-  gnutls_mac_algorithm_t algo;
-  unsigned mac_len, key_len;
-  
+  uint8_t sha_mac[20];
+  uint8_t sha_mac_orig[20];
 
   if (pkcs12 == NULL)
     {
@@ -1032,40 +1028,22 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
       iter = 1;                 /* the default */
     }
 
-  len = sizeof(oid);
-  result =
-    asn1_read_value(pkcs12->pkcs12, "macData.mac.digestAlgorithm.algorithm",
-                    oid, &len);
-  if (result != ASN1_SUCCESS)
-    {
-      gnutls_assert();
-      return _gnutls_asn2err(result);
-    }
-
-  algo = _gnutls_x509_oid_to_mac(oid);
-  if (algo == GNUTLS_MAC_UNKNOWN)
-    {
-      gnutls_assert();
-      return GNUTLS_E_UNKNOWN_HASH_ALGORITHM;
-    }
-
-  mac_len = _gnutls_hmac_get_algo_len(algo);
-  key_len = mac_len;
 
   /* Read the salt from the structure.
    */
   result =
     _gnutls_x509_read_value (pkcs12->pkcs12, "macData.macSalt", &salt);
-  if (result < 0)
+  if (result != ASN1_SUCCESS)
     {
       gnutls_assert ();
+      result = _gnutls_asn2err (result);
       goto cleanup;
     }
 
   /* Generate the key.
    */
-  result = _gnutls_pkcs12_string_to_key (algo, 3 /*MAC*/, salt.data, salt.size,
-                                         iter, pass, key_len, key);
+  result = _gnutls_pkcs12_string_to_key (3 /*MAC*/, salt.data, salt.size,
+                                         iter, pass, sizeof (key), key);
   if (result < 0)
     {
       gnutls_assert ();
@@ -1085,21 +1063,21 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
 
   /* MAC the data
    */
-  result = _gnutls_hmac_init (&td1, algo, key, key_len);
+  result = _gnutls_mac_init (&td1, GNUTLS_MAC_SHA1, key, sizeof (key));
   if (result < 0)
     {
       gnutls_assert ();
       goto cleanup;
     }
 
-  _gnutls_hmac (&td1, tmp.data, tmp.size);
+  _gnutls_mac (&td1, tmp.data, tmp.size);
   _gnutls_free_datum (&tmp);
 
-  _gnutls_hmac_deinit (&td1, mac_output);
+  _gnutls_mac_deinit (&td1, sha_mac);
 
-  len = sizeof (mac_output_orig);
+  len = sizeof (sha_mac_orig);
   result =
-    asn1_read_value (pkcs12->pkcs12, "macData.mac.digest", mac_output_orig,
+    asn1_read_value (pkcs12->pkcs12, "macData.mac.digest", sha_mac_orig,
                      &len);
   if (result != ASN1_SUCCESS)
     {
@@ -1108,8 +1086,7 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
       goto cleanup;
     }
 
-  if ((unsigned)len != mac_len ||
-      memcmp (mac_output_orig, mac_output, len) != 0)
+  if (memcmp (sha_mac_orig, sha_mac, sizeof (sha_mac)) != 0)
     {
       gnutls_assert ();
       return GNUTLS_E_MAC_VERIFY_FAILED;
