@@ -120,7 +120,7 @@ static gnutls_privkey_t pgp_key = NULL;
 static void
 get_keyid (gnutls_openpgp_keyid_t keyid, const char *str)
 {
-  size_t keyid_size = GNUTLS_OPENPGP_KEYID_SIZE;
+  size_t keyid_size = sizeof (keyid);
 
   if (strlen (str) != 16)
     {
@@ -532,9 +532,6 @@ cert_verify_callback (gnutls_session_t session)
           
           fprintf(stderr, "- DANE: %s\n", out.data);
           gnutls_free(out.data);
-
-          if (status != 0 && !insecure)
-            return -1;
         }
 
     }
@@ -632,6 +629,7 @@ init_tls_session (const char *hostname)
 {
   const char *err;
   int ret;
+  unsigned i;
   gnutls_session_t session;
 
   if (priorities == NULL)
@@ -665,6 +663,20 @@ init_tls_session (const char *hostname)
 
   if (HAVE_OPT(DH_BITS)) 
     gnutls_dh_set_prime_bits( session, OPT_VALUE_DH_BITS);
+  
+  if (HAVE_OPT(ALPN))
+    {
+      unsigned proto_n = STACKCT_OPT(ALPN);
+      char** protos  = (void*)STACKLST_OPT(ALPN);
+      gnutls_datum_t p[proto_n];
+      
+      for (i=0;i<proto_n;i++)
+        {
+          p[i].data = (void*)protos[i];
+          p[i].size = strlen(protos[i]);
+        }
+      gnutls_alpn_set_protocols( session, p, proto_n, 0);
+    }
 
   gnutls_credentials_set (session, GNUTLS_CRD_ANON, anon_cred);
   if (srp_cred)
