@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2008, 2010, 2012 Free Software Foundation, Inc.
+ * Copyright (C) 2006-2012 Free Software Foundation, Inc.
  *
  * Author: Simon Josefsson
  *
@@ -293,7 +293,7 @@ doit (void)
   gnutls_datum_t der3Cert = { der3, sizeof (der3) };
   gnutls_x509_crt_t cert;
 
-  ret = gnutls_global_init ();
+  ret = global_init ();
   if (ret < 0)
     fail ("init %d\n", ret);
 
