@@ -147,7 +147,7 @@ doit (void)
   char buf[17];
   size_t buf_size;
 
-  ret = gnutls_global_init ();
+  ret = global_init ();
   if (ret < 0)
     fail ("init %d\n", ret);
 
