@@ -94,7 +94,7 @@ void doit(void)
   int cret = GNUTLS_E_AGAIN;
 
   /* General init. */
-  gnutls_global_init ();
+  global_init ();
   gnutls_global_set_log_function (tls_log_func);
   if (debug)
     gnutls_global_set_log_level (2);
